@@ -1,7 +1,8 @@
 /*
  * userconfig.c -- Read and write the user configuration file
  *
- * Copyright (C) 2007 Alberto Garcia <agarcia@igalia.com>
+ * Copyright (C) 2007-2009 Igalia, S.L.
+ * Authors: Alberto Garcia <agarcia@igalia.com>
  *
  * This file is part of Vagalume and is published under the GNU GPLv3.
  * See the README file for more details.
@@ -16,13 +17,16 @@
 
 #include "globaldefs.h"
 #include "userconfig.h"
+#include "lastfm-ws.h"
+#include "scrobbler.h"
 #include "util.h"
 
 #define DEFAULT_IMSTATUS_TEMPLATE \
         "\342\231\253 {artist} - {title} \342\231\253 (Vagalume {version})"
 
 static char *
-cfg_get_val(const char *line, const char *key)
+cfg_get_val                             (const char *line,
+                                         const char *key)
 {
         g_return_val_if_fail(line != NULL && key != NULL, NULL);
         regex_t creg;
@@ -44,14 +48,14 @@ cfg_get_val(const char *line, const char *key)
 
 /* Old config file, to be removed soon */
 static char *
-get_old_cfg_filename(void)
+get_old_cfg_filename                    (void)
 {
         const char *homedir = get_home_directory ();
         return g_strconcat(homedir, "/.vagalumerc", NULL);
 }
 
 static char *
-default_download_dir(void)
+default_download_dir                    (void)
 {
         const char *homedir = get_home_directory ();
         char *dldir = NULL;
@@ -67,7 +71,7 @@ default_download_dir(void)
 }
 
 const char *
-vgl_user_cfg_get_cfgdir(void)
+vgl_user_cfg_get_cfgdir                 (void)
 {
         static char *cfgdir = NULL;
         if (cfgdir == NULL) {
@@ -84,7 +88,7 @@ vgl_user_cfg_get_cfgdir(void)
 }
 
 static const char *
-get_cfg_filename(void)
+get_cfg_filename                        (void)
 {
         static char *cfgfile = NULL;
 
@@ -96,7 +100,8 @@ get_cfg_filename(void)
 }
 
 void
-vgl_user_cfg_set_username(VglUserCfg *cfg, const char *username)
+vgl_user_cfg_set_username               (VglUserCfg *cfg,
+                                         const char *username)
 {
         g_return_if_fail(cfg != NULL && username != NULL);
         g_free(cfg->username);
@@ -104,15 +109,45 @@ vgl_user_cfg_set_username(VglUserCfg *cfg, const char *username)
 }
 
 void
-vgl_user_cfg_set_password(VglUserCfg *cfg, const char *password)
+vgl_user_cfg_set_password               (VglUserCfg *cfg,
+                                         const char *password)
 {
         g_return_if_fail(cfg != NULL && password != NULL);
         g_free(cfg->password);
         cfg->password = g_strstrip(g_strdup(password));
 }
 
+static void
+vgl_user_cfg_set_server                 (VglUserCfg *cfg,
+                                         VglServer  *server)
+{
+        g_return_if_fail (cfg != NULL && server != NULL);
+        vgl_object_ref (server);
+        if (cfg->server) {
+                vgl_object_unref (cfg->server);
+        }
+        cfg->server = server;
+}
+
 void
-vgl_user_cfg_set_http_proxy(VglUserCfg *cfg, const char *proxy)
+vgl_user_cfg_set_server_name            (VglUserCfg *cfg,
+                                         const char *name)
+{
+        VglServer *server;
+
+        g_return_if_fail (cfg != NULL && name != NULL);
+
+        server = vgl_server_list_find_by_name (name);
+        if (server) {
+                vgl_user_cfg_set_server (cfg, server);
+        } else {
+                g_warning ("No such server: %s", name);
+        }
+}
+
+void
+vgl_user_cfg_set_http_proxy             (VglUserCfg *cfg,
+                                         const char *proxy)
 {
         g_return_if_fail(cfg != NULL && proxy != NULL);
         g_free(cfg->http_proxy);
@@ -120,7 +155,8 @@ vgl_user_cfg_set_http_proxy(VglUserCfg *cfg, const char *proxy)
 }
 
 void
-vgl_user_cfg_set_download_dir(VglUserCfg *cfg, const char *dir)
+vgl_user_cfg_set_download_dir           (VglUserCfg *cfg,
+                                         const char *dir)
 {
         g_return_if_fail(cfg != NULL && dir != NULL);
         g_free(cfg->download_dir);
@@ -128,7 +164,8 @@ vgl_user_cfg_set_download_dir(VglUserCfg *cfg, const char *dir)
 }
 
 void
-vgl_user_cfg_set_imstatus_template(VglUserCfg *cfg, const char *str)
+vgl_user_cfg_set_imstatus_template      (VglUserCfg *cfg,
+                                         const char *str)
 {
         g_return_if_fail(cfg != NULL && str != NULL);
         g_free(cfg->imstatus_template);
@@ -136,7 +173,7 @@ vgl_user_cfg_set_imstatus_template(VglUserCfg *cfg, const char *str)
 }
 
 VglUserCfg *
-vgl_user_cfg_new(void)
+vgl_user_cfg_new                        (void)
 {
         VglUserCfg *cfg = g_slice_new0(VglUserCfg);
         cfg->username = g_strdup("");
@@ -144,6 +181,8 @@ vgl_user_cfg_new(void)
         cfg->http_proxy = g_strdup("");
         cfg->download_dir = default_download_dir();
         cfg->imstatus_template = g_strdup(DEFAULT_IMSTATUS_TEMPLATE);
+        cfg->server = vgl_server_get_default ();
+        if (cfg->server) vgl_object_ref (cfg->server);
         cfg->use_proxy = FALSE;
         cfg->enable_scrobbling = TRUE;
         cfg->discovery_mode = FALSE;
@@ -154,22 +193,24 @@ vgl_user_cfg_new(void)
         cfg->disable_confirm_dialogs = FALSE;
         cfg->show_notifications = TRUE;
         cfg->close_to_systray = TRUE;
+        cfg->autodl_free_tracks = FALSE;
         return cfg;
 }
 
 void
-vgl_user_cfg_destroy(VglUserCfg *cfg)
+vgl_user_cfg_destroy                    (VglUserCfg *cfg)
 {
         g_free(cfg->username);
         g_free(cfg->password);
         g_free(cfg->http_proxy);
         g_free(cfg->download_dir);
         g_free(cfg->imstatus_template);
+        vgl_object_unref(cfg->server);
         g_slice_free(VglUserCfg, cfg);
 }
 
 static VglUserCfg *
-lastfm_old_usercfg_read(void)
+lastfm_old_usercfg_read                 (void)
 {
         VglUserCfg *cfg = NULL;
         const int bufsize = 256;
@@ -235,71 +276,8 @@ lastfm_old_usercfg_read(void)
         return cfg;
 }
 
-static void
-xml_add_string (xmlNode *parent, const char *name, const char *value)
-{
-        xmlNode *node;
-        xmlChar *enc;
-
-        node = xmlNewNode (NULL, (xmlChar *) name);
-        enc = xmlEncodeEntitiesReentrant (NULL, (xmlChar *) value);
-        xmlNodeSetContent (node, enc);
-        xmlFree (enc);
-
-        xmlAddChild (parent, node);
-}
-
-static void
-xml_add_bool (xmlNode *parent, const char *name, gboolean value)
-{
-        xml_add_string (parent, name, value ? "1" : "0");
-}
-
-static void
-xml_get_string (xmlDoc *doc, const xmlNode *node,
-                const char *name, char **value)
-{
-        const xmlNode *iter;
-        xmlChar *val = NULL;
-        gboolean found = FALSE;
-
-        g_return_if_fail (doc && node && name && value);
-
-        for (iter = node; iter != NULL && !found; iter = iter->next) {
-                if (!xmlStrcmp (iter->name, (const xmlChar *) name)) {
-                        val = xmlNodeListGetString
-                                (doc, iter->xmlChildrenNode, 1);
-                        found = TRUE;
-                }
-
-        }
-
-        g_free (*value);
-
-        if (val != NULL) {
-                *value = g_strstrip (g_strdup ((char *) val));
-                xmlFree (val);
-        } else {
-                *value = g_strdup ("");
-        }
-
-}
-
-static void
-xml_get_bool (xmlDoc *doc, const xmlNode *node,
-              const char *name, gboolean *value)
-{
-        char *strval = NULL;
-
-        g_return_if_fail (value != NULL);
-
-        xml_get_string (doc, node, name, &strval);
-        *value = g_str_equal (strval, "1");
-        g_free (strval);
-}
-
 VglUserCfg *
-vgl_user_cfg_read (void)
+vgl_user_cfg_read                       (void)
 {
         xmlDoc *doc = NULL;
         xmlNode *root = NULL;
@@ -331,29 +309,48 @@ vgl_user_cfg_read (void)
                 root = xmlDocGetRootElement (doc);
                 xmlChar *version = xmlGetProp (root, (xmlChar *) "version");
                 if (version == NULL ||
-                    xmlStrcmp (root->name, (const xmlChar *) "config")) {
+                    !xmlStrEqual (root->name, (xmlChar *) "config")) {
                         g_warning ("Error parsing config file");
-                } else if (xmlStrcmp (version, (const xmlChar *) "1")) {
+                } else if (!xmlStrEqual (version, (xmlChar *) "1")) {
                         g_warning ("This config file is version %s, but "
                                    "Vagalume " APP_VERSION " can only "
                                    "read version 1", version);
                 } else {
                         node = root->xmlChildrenNode;
                 }
+                if (version != NULL) xmlFree (version);
         }
 
         /* Parse the configuration */
         if (node != NULL) {
+                char *str;
                 cfg = vgl_user_cfg_new();
                 /* This code is not very optimal, but it is simpler */
-                xml_get_string (doc, node, "username", &(cfg->username));
-                xml_get_string (doc, node, "password", &(cfg->password));
-                obfuscate_string (cfg->password);
-                xml_get_string (doc, node, "http-proxy", &(cfg->http_proxy));
-                xml_get_string (doc, node, "download-dir",
-                                &(cfg->download_dir));
-                xml_get_string (doc, node, "imstatus-template",
-                                &(cfg->imstatus_template));
+                if (xml_get_string (doc, node, "username", &str)) {
+                        vgl_user_cfg_set_username (cfg, str);
+                        g_free (str);
+                }
+                if (xml_get_string (doc, node, "password", &str)) {
+                        obfuscate_string (str);
+                        vgl_user_cfg_set_password (cfg, str);
+                        g_free (str);
+                }
+                if (xml_get_string (doc, node, "http-proxy", &str)) {
+                        vgl_user_cfg_set_http_proxy (cfg, str);
+                        g_free (str);
+                }
+                if (xml_get_string (doc, node, "download-dir", &str)) {
+                        vgl_user_cfg_set_download_dir (cfg, str);
+                        g_free (str);
+                }
+                if (xml_get_string (doc, node, "imstatus-template", &str)) {
+                        vgl_user_cfg_set_imstatus_template (cfg, str);
+                        g_free (str);
+                }
+                if (xml_get_string (doc, node, "server-name", &str)) {
+                        vgl_user_cfg_set_server_name (cfg, str);
+                        g_free (str);
+                }
                 xml_get_bool (doc, node, "use-proxy", &(cfg->use_proxy));
                 xml_get_bool (doc, node, "discovery-mode",
                               &(cfg->discovery_mode));
@@ -369,6 +366,8 @@ vgl_user_cfg_read (void)
                               &(cfg->show_notifications));
                 xml_get_bool (doc, node, "close-to-systray",
                               &(cfg->close_to_systray));
+                xml_get_bool (doc, node, "autodownload-free-tracks",
+                              &(cfg->autodl_free_tracks));
         }
 
         if (doc != NULL) xmlFreeDoc (doc);
@@ -377,7 +376,7 @@ vgl_user_cfg_read (void)
 }
 
 gboolean
-vgl_user_cfg_write (VglUserCfg *cfg)
+vgl_user_cfg_write                      (VglUserCfg *cfg)
 {
         xmlDoc *doc;
         xmlNode *root;
@@ -390,12 +389,13 @@ vgl_user_cfg_write (VglUserCfg *cfg)
         doc = xmlNewDoc ((xmlChar *) "1.0");
         root = xmlNewNode (NULL, (xmlChar *) "config");
         xmlSetProp (root, (xmlChar *) "version", (xmlChar *) "1");
-        xmlSetProp (root, (xmlChar *) "revision", (xmlChar *) "1");
+        xmlSetProp (root, (xmlChar *) "revision", (xmlChar *) "3");
         xmlDocSetRootElement (doc, root);
 
         xml_add_string (root, "username", cfg->username);
         xml_add_string (root, "password", obfuscate_string (cfg->password));
         obfuscate_string (cfg->password);
+        xml_add_string (root, "server-name", cfg->server->name);
         xml_add_string (root, "http-proxy", cfg->http_proxy);
         xml_add_string (root, "download-dir", cfg->download_dir);
         xml_add_string (root, "imstatus-template", cfg->imstatus_template);
@@ -410,6 +410,8 @@ vgl_user_cfg_write (VglUserCfg *cfg)
                       cfg->disable_confirm_dialogs);
         xml_add_bool (root, "show-notifications", cfg->show_notifications);
         xml_add_bool (root, "close-to-systray", cfg->close_to_systray);
+        xml_add_bool (root, "autodownload-free-tracks",
+                      cfg->autodl_free_tracks);
 
         if (xmlSaveFormatFileEnc (cfgfile, doc, "UTF-8", 1) == -1) {
                 g_critical ("Unable to open %s", cfgfile);
